@@ -1,5 +1,7 @@
-from django.forms import ModelForm, Field, ValidationError
+from django.forms import ModelForm, Field, ValidationError, BooleanField
+from django.forms.widgets import CheckboxInput
 from explorer.models import Query, MSG_FAILED_BLACKLIST
+from django.db import DatabaseError
 
 _ = lambda x: x
 
@@ -15,10 +17,15 @@ class SqlField(Field):
 
         query = Query(sql=value)
 
-        error = MSG_FAILED_BLACKLIST if not query.passes_blacklist() else None
+        passes_blacklist, failing_words = query.passes_blacklist()
+
+        error = MSG_FAILED_BLACKLIST % ', '.join(failing_words) if not passes_blacklist else None
 
         if not error and not query.available_params():
-            error = query.try_execute()
+            try:
+                query.execute_query_only()
+            except DatabaseError as e:
+                error = str(e)
 
         if error:
             raise ValidationError(
@@ -30,6 +37,7 @@ class SqlField(Field):
 class QueryForm(ModelForm):
 
     sql = SqlField()
+    snapshot = BooleanField(widget=CheckboxInput, required=False)
 
     def clean(self):
         if self.instance and self.data.get('created_by_user', None):
@@ -38,12 +46,12 @@ class QueryForm(ModelForm):
 
     @property
     def created_by_user_email(self):
-        return self.instance.created_by_user.email
+        return self.instance.created_by_user.email if self.instance.created_by_user else '--'
 
     @property
     def created_by_user_id(self):
-        return self.instance.created_by_user.id
+        return self.instance.created_by_user.id if self.instance.created_by_user else ''
 
     class Meta:
         model = Query
-        fields = ['title', 'sql', 'description', 'created_by_user']
\ No newline at end of file
+        fields = ['title', 'sql', 'description', 'created_by_user', 'snapshot']
\ No newline at end of file
