@@ -20,4 +20,10 @@ class TestFormValidation(TestCase):
         q = SimpleQueryFactory(sql="select $$a$$;", created_by_user_id=None)
         q.params = {}
         form = QueryForm(model_to_dict(q))
-        self.assertTrue(form.is_valid())
\ No newline at end of file
+        self.assertTrue(form.is_valid())
+
+    def test_form_fails_blacklist_even_with_params(self):
+        q = SimpleQueryFactory(sql="delete $$a$$;", created_by_user_id=None)
+        q.params = {}
+        form = QueryForm(model_to_dict(q))
+        self.assertFalse(form.is_valid())
\ No newline at end of file
